-- Lesson: SIR model - Simulating communicable disease transmission through individual 
--         and finite resources limit the system growth. 
-- Authors:  Tiago Carneiro & Gilberto Camara

---------------------------------------------------------------
-- PARAMETER
---------------------------------------------------------------
TOTAL 										= 1000
CONTACTS_PER_INFECTION_DAY	= 1
CONTAGION_STRENGTH			 	= 0.4
INFECTIOUS_PERIOD					= 4

---------------------------------------------------------------
-- GLOBAL VARIABLES
---------------------------------------------------------------
infectionRate	= 0 
recoveryRate	= 0
R0				= 0

---------------------------------------------------------------
-- MODEL
---------------------------------------------------------------
system = Agent { 
	susceptible = 999,
	infected = 1, 
	recovered = 0, 
	execute = function( self )
			local change_infected  = infectionRate*self.susceptible*self.infected
		
			local change_recovered = recoveryRate*self.infected
			
			if change_infected > self.susceptible then
				change_infected = self.susceptible
			end		

			self.susceptible	= self.susceptible 	- change_infected 
			self.infected		= self.infected 		+ change_infected - change_recovered
			self.recovered		= self.recovered 		+ change_recovered
	end
} 


function calcParameters ( )

	infectionRate = CONTACTS_PER_INFECTION_DAY * CONTAGION_STRENGTH / TOTAL

	recoveryRate = 1 / INFECTIOUS_PERIOD

	R0 = (infectionRate * TOTAL) / recoveryRate

	print ("R0   = ", R0)
end 

---------------------------------------------------------------
-- SIMULATION
---------------------------------------------------------------
chart = Observer {
	subject     	= system,
	attributes  	= {"susceptible",  "infected", "recovered"},
	curveLabels = {"susceptible",  "infected", "recovered"},
	title       	= "SIR model",
	xLabel      	= "days",
	yLabel      	= "people",
	type = "chart"
}

timer = Timer {
	Event{time = 1, period = 1, action = function(ev)
		system:execute() 
		system:notify(ev:getTime()) 
	end}
}

-- Calculate values of global constants from parameters
calcParameters (model)

timer:execute (100)



